@@ -1,12 +1,17 @@
+use std::time::Duration;
 use stretto::{Cache, DefaultKeyBuilder};
 
 #[tokio::main]
 async fn main() {
-    let c: Cache<&str, &str, DefaultKeyBuilder> = Cache::new(12960, 1e6 as i64, DefaultKeyBuilder::default()).unwrap();
+    let c: Cache<&str, &str, DefaultKeyBuilder> =
+        Cache::new(12960, 1e6 as i64, DefaultKeyBuilder::default()).unwrap();
 
     // set a value with a cost of 1
     c.insert("a", "a", 1).await;
 
+    // set a value with a cost of 1 and ttl
+    c.insert_with_ttl("b", "b", 1, Duration::from_secs(3)).await;
+
     // wait for value to pass through buffers
     c.wait().await.unwrap();
 
@@ -40,4 +45,4 @@ async fn main() {
     c.wait().await.unwrap();
 
     assert!(c.get(&"a").is_none());
-}
\ No newline at end of file
+}
