@@ -2,17 +2,21 @@
 mod test;
 mod wg;
 
-cfg_async!(use crate::sleep;);
-cfg_not_async!(use crossbeam_channel::{tick, RecvError};);
+cfg_async!(
+    use crate::sleep;
+);
+cfg_not_async!(
+    use crossbeam_channel::{tick, RecvError};
+);
 
 use crate::{
     bounded,
-    policy::LFUPolicy,
-    ttl::ExpirationMap,
     cache::wg::WaitGroup,
     metrics::MetricType,
+    policy::LFUPolicy,
     select, spawn, stop_channel,
     store::{ShardedMap, UpdateResult},
+    ttl::ExpirationMap,
     ttl::Time,
     unbounded,
     utils::{ValueRef, ValueRefMut},
@@ -20,9 +24,9 @@ use crate::{
     Instant, Item as CrateItem, JoinHandle, KeyBuilder, Metrics, Receiver, Sender,
     UnboundedReceiver, UnboundedSender, UpdateValidator,
 };
-use std::hash::{BuildHasher, Hash};
 use std::collections::hash_map::RandomState;
 use std::collections::HashMap;
+use std::hash::{BuildHasher, Hash};
 use std::marker::PhantomData;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
@@ -89,27 +93,27 @@ impl<V> Item<V> {
 }
 
 /// The `CacheBuilder` struct is used when creating Cache instances if you want to customize the Cache settings.
-/// 
+///
 /// - **num_counters**
 ///
 ///     `num_counters` is the number of 4-bit access counters to keep for admission and eviction.
 ///     Dgraph's developers have seen good performance in setting this to 10x the number of items
 ///     you expect to keep in the cache when full.
-/// 
+///
 ///     For example, if you expect each item to have a cost of 1 and `max_cost` is 100, set `num_counters` to 1,000.
 ///     Or, if you use variable cost values but expect the cache to hold around 10,000 items when full,
 ///     set num_counters to 100,000. The important thing is the *number of unique items* in the full cache,
 ///     not necessarily the `max_cost` value.
-/// 
+///
 /// - **max_cost**
-/// 
+///
 ///     `max_cost` is how eviction decisions are made. For example, if max_cost is 100 and a new item
 ///     with a cost of 1 increases total cache cost to 101, 1 item will be evicted.
-/// 
+///
 ///     `max_cost` can also be used to denote the max size in bytes. For example,
 ///     if max_cost is 1,000,000 (1MB) and the cache is full with 1,000 1KB items,
 ///     a new item (that's accepted) would cause 5 1KB items to be evicted.
-/// 
+///
 ///     `max_cost` could be anything as long as it matches how you're using the cost values when calling [`insert`].
 ///
 /// - **key_builder**
@@ -125,18 +129,18 @@ impl<V> Item<V> {
 ///     any 64bit hash.
 ///
 /// - **buffer_size**
-/// 
+///
 ///     `buffer_size` is the size of the insert buffers. The Dgraph's developers find that 32 * 1024 gives a good performance.
-/// 
+///
 ///     If for some reason you see insert performance decreasing with lots of contention (you shouldn't),
 ///     try increasing this value in increments of 32 * 1024. This is a fine-tuning mechanism
 ///     and you probably won't have to touch this.
-/// 
+///
 /// - **metrics**
-/// 
+///
 ///     Metrics is true when you want real-time logging of a variety of stats.
 ///     The reason this is a [`CacheBuilder`] flag is because there's a 10% throughput performance overhead.
-/// 
+///
 /// - **ignore_internal_cost**
 ///
 ///     Set to true indicates to the cache that the cost of
@@ -149,24 +153,24 @@ impl<V> Item<V> {
 ///     The Cache will cleanup the expired values every 500ms by default.
 ///
 /// - **update_validator**
-/// 
+///
 ///     By default, the Cache will always update the value if the value already exists in the cache.
 ///     [`UpdateValidator`] is a trait to support customized update policy (check if the value should be updated
 ///     if the value already exists in the cache).
 ///
 /// - **callback**
-/// 
+///
 ///     [`CacheCallback`] is for customize some extra operations on values when related event happens..
 ///
 /// - **coster**
-/// 
+///
 ///     [`Coster`] is a trait you can pass to the [`CacheBuilder`] in order to evaluate
 ///     item cost at runtime, and only for the [`insert`] calls that aren't dropped (this is
 ///     useful if calculating item cost is particularly expensive, and you don't want to
 ///     waste time on items that will be dropped anyways).
-/// 
+///
 ///     To signal to Stretto that you'd like to use this Coster trait:
-/// 
+///
 ///     1. Set the Coster field to your own Coster implementation.
 ///     2. When calling [`insert`] for new items or item updates, use a cost of 0.
 ///
@@ -248,13 +252,13 @@ pub struct CacheBuilder<
 /// policy and a Sampled LFU eviction policy. You can use the same Cache instance
 /// from as many threads as you want.
 ///
-/// 
+///
 /// # Features
-/// * **Internal Mutability** - Do not need to use `Arc<RwLock<Cache<...>>` for concurrent code, you just need `Arc<Cache<...>`
+/// * **Internal Mutability** - Do not need to use `Arc<RwLock<Cache<...>>` for concurrent code, you just need `Cache<...>`
 /// * **Sync and Async** - Stretto support async by `tokio` and sync by `crossbeam`.
 ///   * In sync, Cache starts two extra OS level threads. One is policy thread, the other is writing thread.
 ///   * In async, Cache starts two extra green threads. One is policy thread, the other is writing thread.
-/// * **Store policy** Stretto only store the value, which means the cache does not store the key. 
+/// * **Store policy** Stretto only store the value, which means the cache does not store the key.
 /// * **High Hit Ratios** - with our unique admission/eviction policy pairing, Ristretto's performance is best in class.
 ///     * **Eviction: SampledLFU** - on par with exact LRU and better performance on Search and Database traces.
 ///     * **Admission: TinyLFU** - extra performance with little memory overhead (12 bits per counter).
@@ -263,7 +267,7 @@ pub struct CacheBuilder<
 /// * **Fully Concurrent** - you can use as many threads as you want with little throughput degradation.
 /// * **Metrics** - optional performance metrics for throughput, hit ratios, and other stats.
 /// * **Simple API** - just figure out your ideal [`CacheBuilder`] values and you're off and running.
-/// 
+///
 /// [`CacheBuilder`]: struct.CacheBuilder.html
 pub struct Cache<
     K,
@@ -294,9 +298,9 @@ pub struct Cache<
 
     callback: Arc<CB>,
 
-    key_to_hash: KH,
+    key_to_hash: Arc<KH>,
 
-    is_closed: AtomicBool,
+    is_closed: Arc<AtomicBool>,
 
     coster: Arc<C>,
 
@@ -307,11 +311,11 @@ pub struct Cache<
 }
 
 struct CacheProcessor<V, U, CB, S>
-    where
-        V: Send + Sync + 'static,
-        U: UpdateValidator<V>,
-        CB: CacheCallback<V>,
-        S: BuildHasher + Clone + 'static,
+where
+    V: Send + Sync + 'static,
+    U: UpdateValidator<V>,
+    CB: CacheCallback<V>,
+    S: BuildHasher + Clone + 'static,
 {
     insert_buf_rx: Receiver<Item<V>>,
     stop_rx: Receiver<()>,
@@ -328,25 +332,25 @@ struct CacheProcessor<V, U, CB, S>
 }
 
 struct CacheCleaner<'a, V, U, CB, S>
-    where
-        V: Send + Sync + 'static,
-        U: UpdateValidator<V>,
-        CB: CacheCallback<V>,
-        S: BuildHasher + Clone + 'static,
+where
+    V: Send + Sync + 'static,
+    U: UpdateValidator<V>,
+    CB: CacheCallback<V>,
+    S: BuildHasher + Clone + 'static,
 {
     processor: &'a mut CacheProcessor<V, U, CB, S>,
 }
 
 impl<K: Hash + Eq, V: Send + Sync + 'static, KH: KeyBuilder<K>>
-CacheBuilder<
-    K,
-    V,
-    KH,
-    DefaultCoster<V>,
-    DefaultUpdateValidator<V>,
-    DefaultCacheCallback<V>,
-    RandomState,
->
+    CacheBuilder<
+        K,
+        V,
+        KH,
+        DefaultCoster<V>,
+        DefaultUpdateValidator<V>,
+        DefaultCacheCallback<V>,
+        RandomState,
+    >
 {
     /// Create a new CacheBuilder
     #[inline]
@@ -371,14 +375,14 @@ CacheBuilder<
 }
 
 impl<K, V, KH, C, U, CB, S> CacheBuilder<K, V, KH, C, U, CB, S>
-    where
-        K: Hash + Eq,
-        V: Send + Sync + 'static,
-        KH: KeyBuilder<K>,
-        C: Coster<V>,
-        U: UpdateValidator<V>,
-        CB: CacheCallback<V>,
-        S: BuildHasher + Clone + 'static,
+where
+    K: Hash + Eq,
+    V: Send + Sync + 'static,
+    KH: KeyBuilder<K>,
+    C: Coster<V>,
+    U: UpdateValidator<V>,
+    CB: CacheCallback<V>,
+    S: BuildHasher + Clone + 'static,
 {
     /// Set the number of counters for the Cache.
     ///
@@ -581,10 +585,7 @@ impl<K, V, KH, C, U, CB, S> CacheBuilder<K, V, KH, C, U, CB, S>
     /// 1. Set the [`Coster`] field to your own [`Coster`] implementation.
     /// 2. When calling [`insert`] for new items or item updates, use a cost of 0.
     #[inline]
-    pub fn set_coster<NC: Coster<V>>(
-        self,
-        coster: NC,
-    ) -> CacheBuilder<K, V, KH, NC, U, CB, S> {
+    pub fn set_coster<NC: Coster<V>>(self, coster: NC) -> CacheBuilder<K, V, KH, NC, U, CB, S> {
         CacheBuilder {
             num_counters: self.num_counters,
             max_cost: self.max_cost,
@@ -701,7 +702,6 @@ impl<K, V, KH, C, U, CB, S> CacheBuilder<K, V, KH, C, U, CB, S>
         let (stop_tx, stop_rx) = stop_channel();
         let (clear_tx, clear_rx) = unbounded();
 
-
         let hasher = self.hasher.unwrap();
         let expiration_map = ExpirationMap::with_hasher(hasher.clone());
 
@@ -711,8 +711,7 @@ impl<K, V, KH, C, U, CB, S> CacheBuilder<K, V, KH, C, U, CB, S>
             hasher.clone(),
         ));
 
-        let mut policy =
-            LFUPolicy::with_hasher(num_counters, max_cost, hasher.clone())?;
+        let mut policy = LFUPolicy::with_hasher(num_counters, max_cost, hasher.clone())?;
 
         let coster = Arc::new(self.coster.unwrap());
         let callback = Arc::new(self.callback.unwrap());
@@ -736,17 +735,18 @@ impl<K, V, KH, C, U, CB, S> CacheBuilder<K, V, KH, C, U, CB, S>
             clear_rx,
             metrics.clone(),
             callback.clone(),
-        ).spawn();
+        )
+        .spawn();
 
         let this = Cache {
             store,
             policy,
             insert_buf_tx: buf_tx,
             callback,
-            key_to_hash: self.key_to_hash,
+            key_to_hash: Arc::new(self.key_to_hash),
             stop_tx,
             clear_tx,
-            is_closed: AtomicBool::new(false),
+            is_closed: Arc::new(AtomicBool::new(false)),
             coster,
             metrics,
             _marker: Default::default(),
@@ -785,14 +785,14 @@ impl<K: Hash + Eq, V: Send + Sync + 'static, KH: KeyBuilder<K>> Cache<K, V, KH>
 }
 
 impl<K, V, KH, C, U, CB, S> Cache<K, V, KH, C, U, CB, S>
-    where
-        K: Hash + Eq,
-        V: Send + Sync + 'static,
-        KH: KeyBuilder<K>,
-        C: Coster<V>,
-        U: UpdateValidator<V>,
-        CB: CacheCallback<V>,
-        S: BuildHasher + Clone + 'static,
+where
+    K: Hash + Eq,
+    V: Send + Sync + 'static,
+    KH: KeyBuilder<K>,
+    C: Coster<V>,
+    U: UpdateValidator<V>,
+    CB: CacheCallback<V>,
+    S: BuildHasher + Clone + 'static,
 {
     /// `get` returns a `Option<ValueRef<V, SS>>` (if any) representing whether the
     /// value was found or not.
@@ -1155,12 +1155,39 @@ impl<K, V, KH, C, U, CB, S> Cache<K, V, KH, C, U, CB, S>
     }
 }
 
-impl<V, U, CB, S> CacheProcessor<V, U, CB, S>
+impl<K, V, KH, C, U, CB, S> Clone for Cache<K, V, KH, C, U, CB, S>
     where
+        K: Hash + Eq,
         V: Send + Sync + 'static,
+        KH: KeyBuilder<K>,
+        C: Coster<V>,
         U: UpdateValidator<V>,
         CB: CacheCallback<V>,
         S: BuildHasher + Clone + 'static,
+{
+    fn clone(&self) -> Self {
+        Self {
+            store: self.store.clone(),
+            policy: self.policy.clone(),
+            insert_buf_tx: self.insert_buf_tx.clone(),
+            stop_tx: self.stop_tx.clone(),
+            clear_tx: self.clear_tx.clone(),
+            callback: self.callback.clone(),
+            key_to_hash: self.key_to_hash.clone(),
+            is_closed: self.is_closed.clone(),
+            coster: self.coster.clone(),
+            metrics: self.metrics.clone(),
+            _marker: self._marker,
+        }
+    }
+}
+
+impl<V, U, CB, S> CacheProcessor<V, U, CB, S>
+where
+    V: Send + Sync + 'static,
+    U: UpdateValidator<V>,
+    CB: CacheCallback<V>,
+    S: BuildHasher + Clone + 'static,
 {
     fn new(
         num_to_keep: usize,
@@ -1187,7 +1214,7 @@ impl<V, U, CB, S> CacheProcessor<V, U, CB, S>
             callback,
             ignore_internal_cost,
             item_size,
-            cleanup_duration
+            cleanup_duration,
         }
     }
 
@@ -1394,15 +1421,14 @@ impl<V, U, CB, S> CacheProcessor<V, U, CB, S>
             self.start_ts.remove(&item.index);
         }
     }
-
 }
 
 impl<'a, V, U, CB, S> CacheCleaner<'a, V, U, CB, S>
-    where
-        V: Send + Sync + 'static,
-        U: UpdateValidator<V>,
-        CB: CacheCallback<V>,
-        S: BuildHasher + Clone + 'static,
+where
+    V: Send + Sync + 'static,
+    U: UpdateValidator<V>,
+    CB: CacheCallback<V>,
+    S: BuildHasher + Clone + 'static,
 {
     #[inline]
     fn new(processor: &'a mut CacheProcessor<V, U, CB, S>) -> Self {
@@ -1418,13 +1444,13 @@ impl<'a, V, U, CB, S> CacheCleaner<'a, V, U, CB, S>
                 cost,
                 value,
                 expiration,
-            } => self.processor.callback.on_evict(CrateItem {
-                val: Some(value),
-                index: key,
+            } => self.processor.callback.on_evict(CrateItem::new(
+                key,
                 conflict,
                 cost,
-                exp: expiration,
-            }),
+                Some(value),
+                expiration,
+            )),
             Item::Delete { .. } | Item::Update { .. } => {}
             Item::Wait(wg) => drop(wg),
         }
@@ -1459,4 +1485,4 @@ impl<'a, V, U, CB, S> CacheCleaner<'a, V, U, CB, S>
             }
         }
     }
-}
\ No newline at end of file
+}
