@@ -1,14 +1,17 @@
 use crate::cache::{Cache, Item};
-use crate::{CacheCallback, Coster, DefaultKeyBuilder, Item as CrateItem, KeyBuilder, TransparentKeyBuilder, UpdateValidator, CacheBuilder, DefaultCoster, DefaultUpdateValidator, DefaultCacheCallback};
+use crate::{
+    CacheBuilder, CacheCallback, Coster, DefaultCacheCallback, DefaultCoster, DefaultKeyBuilder,
+    DefaultUpdateValidator, Item as CrateItem, KeyBuilder, TransparentKeyBuilder, UpdateValidator,
+};
 use parking_lot::Mutex;
 use rand::rngs::OsRng;
 use rand::Rng;
+use std::collections::hash_map::RandomState;
 use std::collections::HashSet;
 use std::hash::Hash;
 use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::Arc;
 use std::time::Duration;
-use std::collections::hash_map::RandomState;
 
 static CHARSET: &'static [u8] = "abcdefghijklmnopqrstuvwxyz0123456789".as_bytes();
 
@@ -23,14 +26,6 @@ struct TestCallback {
     evicted: Arc<Mutex<HashSet<u64>>>,
 }
 
-impl Default for TestCallback {
-    fn default() -> Self {
-        Self {
-            evicted: Arc::new(Mutex::new(HashSet::new())),
-        }
-    }
-}
-
 impl TestCallback {
     fn new(map: Arc<Mutex<HashSet<u64>>>) -> Self {
         Self { evicted: map }
@@ -108,7 +103,7 @@ cfg_not_async! {
     }
 
     fn retry_set<C: Coster<u64>, U: UpdateValidator<u64>, CB: CacheCallback<u64>>(
-        c: Arc<Cache<u64, u64, TransparentKeyBuilder<u64>, C, U, CB>>,
+        c: Cache<u64, u64, TransparentKeyBuilder<u64>, C, U, CB>,
         key: u64,
         val: u64,
         cost: i64,
@@ -171,7 +166,7 @@ cfg_not_async! {
             .finalize()
             .unwrap();
 
-        let c = Arc::new(c);
+        
         let (stop_tx, stop_rx) = bounded::<()>(8);
 
         for _ in 0..8 {
@@ -213,7 +208,7 @@ cfg_not_async! {
             eprintln!("{}", c.metrics);
             assert!(cost as f64 <= (1e6 * 1.05));
         }
-
+        c.wait().unwrap();
         for _ in 0..8 {
             let _ = stop_tx.send(());
         }
@@ -353,7 +348,7 @@ cfg_not_async! {
             .set_metrics(true)
             .finalize()
             .unwrap();
-        let c = Arc::new(c);
+        
 
         retry_set(c.clone(), 1, 1, 1, Duration::ZERO);
 
@@ -416,13 +411,11 @@ cfg_not_async! {
     #[test]
     fn test_cache_set_with_ttl() {
         let cb = Arc::new(Mutex::new(HashSet::new()));
-        let c = Arc::new(
-            Cache::builder(100, 10, TransparentKeyBuilder::default())
+        let c = Cache::builder(100, 10, TransparentKeyBuilder::default())
                 .set_callback(TestCallback::new(cb.clone()))
                 .set_ignore_internal_cost(true)
                 .finalize()
-                .unwrap(),
-        );
+                .unwrap();
 
         retry_set(c.clone(), 1, 1, 1, Duration::from_secs(1));
 
@@ -464,12 +457,10 @@ cfg_not_async! {
 
     #[test]
     fn test_cache_remove_with_ttl() {
-        let c = Arc::new(
-            Cache::builder(100, 10, TransparentKeyBuilder::default())
+        let c = Cache::builder(100, 10, TransparentKeyBuilder::default())
                 .set_ignore_internal_cost(true)
                 .finalize()
-                .unwrap(),
-        );
+                .unwrap();
 
         retry_set(c.clone(), 3, 1, 1, Duration::from_secs(10));
         sleep(Duration::from_secs(1));
@@ -483,13 +474,11 @@ cfg_not_async! {
 
     #[test]
     fn test_cache_get_ttl() {
-        let c = Arc::new(
-            Cache::builder(100, 10, TransparentKeyBuilder::default())
+        let c = Cache::builder(100, 10, TransparentKeyBuilder::default())
                 .set_metrics(true)
                 .set_ignore_internal_cost(true)
                 .finalize()
-                .unwrap(),
-        );
+                .unwrap();
 
         // try expiration with valid ttl item
         {
@@ -535,7 +524,7 @@ cfg_not_async! {
             .set_ignore_internal_cost(true)
             .finalize()
             .unwrap();
-        let c = Arc::new(c);
+        
         let (stop_tx, stop_rx) = bounded(0);
         let tc = c.clone();
 
@@ -596,7 +585,7 @@ cfg_not_async! {
             .finalize()
             .unwrap();
 
-        let c = Arc::new(c);
+        
         c.insert(1, 1, 1);
 
         let (stop_tx, stop_rx) = bounded(0);
@@ -699,7 +688,7 @@ cfg_async! {
     }
 
     async fn retry_set<C: Coster<u64>, U: UpdateValidator<u64>, CB: CacheCallback<u64>>(
-        c: Arc<Cache<u64, u64, TransparentKeyBuilder<u64>, C, U, CB>>,
+        c: Cache<u64, u64, TransparentKeyBuilder<u64>, C, U, CB>,
         key: u64,
         val: u64,
         cost: i64,
@@ -892,7 +881,7 @@ cfg_async! {
             .set_metrics(true)
             .finalize()
             .unwrap();
-        let c = Arc::new(c);
+        
 
         retry_set(c.clone(), 1, 1, 1, Duration::ZERO).await;
 
@@ -955,13 +944,11 @@ cfg_async! {
     #[tokio::test]
     async fn test_cache_set_with_ttl() {
         let cb = Arc::new(Mutex::new(HashSet::new()));
-        let c = Arc::new(
-            Cache::builder(100, 10, TransparentKeyBuilder::default())
+        let c = Cache::builder(100, 10, TransparentKeyBuilder::default())
                 .set_callback(TestCallback::new(cb.clone()))
                 .set_ignore_internal_cost(true)
                 .finalize()
-                .unwrap(),
-        );
+                .unwrap();
 
         retry_set(c.clone(), 1, 1, 1, Duration::from_secs(1)).await;
 
@@ -1003,12 +990,10 @@ cfg_async! {
 
     #[tokio::test]
     async fn test_cache_remove_with_ttl() {
-        let c = Arc::new(
-            Cache::builder(100, 10, TransparentKeyBuilder::default())
+        let c = Cache::builder(100, 10, TransparentKeyBuilder::default())
                 .set_ignore_internal_cost(true)
                 .finalize()
-                .unwrap(),
-        );
+                .unwrap();
 
         retry_set(c.clone(), 3, 1, 1, Duration::from_secs(10)).await;
         sleep(Duration::from_secs(1)).await;
@@ -1022,13 +1007,11 @@ cfg_async! {
 
     #[tokio::test]
     async fn test_cache_get_ttl() {
-        let c = Arc::new(
-            Cache::builder(100, 10, TransparentKeyBuilder::default())
+        let c = Cache::builder(100, 10, TransparentKeyBuilder::default())
                 .set_metrics(true)
                 .set_ignore_internal_cost(true)
                 .finalize()
-                .unwrap(),
-        );
+                .unwrap();
 
         // try expiration with valid ttl item
         {
@@ -1094,7 +1077,7 @@ cfg_async! {
             .finalize()
             .unwrap();
 
-        let c = Arc::new(c);
+        
         c.insert(1, 1, 1).await;
 
         let (stop_tx, mut stop_rx) = channel(1);
@@ -1196,8 +1179,6 @@ cfg_async! {
             .finalize()
             .unwrap();
 
-        let c = Arc::new(c);
-
         let mut txs = Vec::new();
         let mut rxs = Vec::new();
 
@@ -1219,7 +1200,7 @@ cfg_async! {
                 eprintln!("{}", tc.metrics);
                 assert!(cost as f64 <= (1e6 * 1.05));
             }
-
+            tc.wait().await.unwrap();
             for tx in txs {
                 let _ = tx.send(()).await;
             }
@@ -1260,7 +1241,7 @@ cfg_async! {
             .set_ignore_internal_cost(true)
             .finalize()
             .unwrap();
-        let c = Arc::new(c);
+        
         let (stop_tx, mut stop_rx) = channel(1);
         let tc = c.clone();
         let ctr = Arc::new(AtomicU64::new(0));
@@ -1293,4 +1274,4 @@ cfg_async! {
             assert!(c.get(&i).is_none());
         })
     }
-}
\ No newline at end of file
+}
